# vim filecoding=utf-8
# vi:ts=4:et
# $Id: test_young.py,v 1.2 2004/04/11 03:31:27 etale Exp $
# =====================================================

"""
unittest for young.py
"""
import sets
import unittest
from test import test_support

from young import *

class TestYoung(unittest.TestCase):
    def test_total(self):
        for i in range(1, 6):
            self.assertEqual(young(i).get_square(), factorial(i))

    def test_representation(self):
        y = young(3)
        repr(y)
        str(y)

    def test_up(self):
        expectations = (
            [(1,1,1,1), ((2, 1, 1, 1), (1, 1, 1, 1, 1))],
            [(2,1,1), ((3, 1, 1), (2, 2, 1), (2, 1, 1, 1))],
            [(2,2), ((3, 2), (2, 2, 1))],
            [(3,1), ((4, 1), (3, 2), (3, 1, 1))],
            [(4,), ((5,), (4, 1))],
            )

        for partition, expectation in expectations:
            y = young(partition)
            y.up()
            self.assertEqual(sets.Set(y.get_partition()), sets.Set(expectation))

    def test_down(self):
        expectations = (
            [(1,1,1,1), ((1,1,1),)],
            [(2,1,1), ((1,1,1), (2,1))],
            [(2,2), ((2,1),)],
            [(3,1), ((2,1), (3,))],
            [(4,), ((3,),)],
            )

        for partition, expectation in expectations:
            p = Partition(partition)
            y = young(partition)
            y.down()
            self.assertEqual(sets.Set(y.get_partition()), sets.Set(expectation))


    def test_number_of_up(self):
        expectations = ((2,2), (4,6), (10,24), (26,120), (76,720))

        y = young(2)
        for expectation in expectations:
            self.assertEqual((y.size(), y.get_square()), expectation)
            y.up()

    def test_number_of_down(self):
        expectations = ((76, 720), (26, 120), (10, 24), (4, 6), (2, 2))

        y = young(7)
        for expectation in expectations:
            y.down()
            self.assertEqual((y.size(), y.get_square()), expectation)

    def test_generate(self):
        expectation = (
            [[1], [2], [3], [4]],
            [[1, 2], [3], [4]],
            [[1, 3], [2], [4]],
            [[1, 4], [2], [3]],
            [[1, 2], [3, 4]],
            [[1, 3], [2, 4]],
            [[1, 2, 3], [4]],
            [[1, 2, 4], [3]],
            [[1, 3, 4], [2]],
            [[1, 2, 3, 4]],
            )

        y = young(4)
        self.assertEqual(len(y), len(expectation))
        for tableaux in y:
            self.assert_(tableaux in expectation)

    def test_partition(self):
        for i in range(1, 5):
            set_of_part = sets.Set(partition(i))
            set_of_young = sets.Set(young(i).get_partition())
            self.assertEqual(set_of_part, set_of_young)


class TestBump(unittest.TestCase):
    def test_bump1(self):
        # Fulton p.12
        insert = Young([[1,3], [2]])

        y = Young([[1,2,2,3], [2,3,5,5],[4,4,6],[5,6]])

        expectations = [
            [[1,2,2,2], [2,3,3,5],[4,4,5],[5,6,6]],
            [[1,1,2,2], [2,2,3,5],[3,4,5],[4,6,6],[5]],
            [[1,1,2,2,3], [2,2,3,5],[3,4,5],[4,6,6],[5]],
        ]

        for box, expectation in zip((2,1,3), expectations):
            y.bump(box)
            self.assertEqual(y, expectation)

    def test_bump2(self):
        # Fulton p.37
        y = Young() 
        word = [5,4,8,2,3,4,1,7,5,3,1]
        for w in word:
            y.bump(w)

        self.assertEqual(y, Young([[1,1,3,5],[2,3],[4,4],[5,7],[8]]))

    def test_bump3(self):
        # Fulton p.40
        y = Young()
        word = (1,2,2,1,2,1,1,1,2)
        for w in word:
            y.bump(w)

        self.assertEqual(y, Young([[1,1,1,1,1,2],[2,2,2]]))

    def test_bump_multi(self):
        # Fulton p.12
        young_to_insert = Young([[1,3], [2]])
        young_to_be_inserted = Young([[1,2,2,3], [2,3,5,5],[4,4,6],[5,6]])

        # young_to_be_inserted <- young_to_insert
        young_to_be_inserted*young_to_insert

        self.assertEqual(young_to_be_inserted, Young([[1,1,2,2,3], [2,2,3,5],[3,4,5],[4,6,6],[5]]))

    def test_bump_hotta1(self):
        # Hotta p.120
        y = Young()
        word = (3,2,6,1,4,5)
        for w in word:
            y.bump(w)

        self.assertEqual(y, Young([[1,4,5],[2,6],[3]]))

    def test_bump_hotta2(self):
        # Hotta p.120
        y = Young()
        word = (5,3,4,7,1,2,6)
        for w in word:
            y.bump(w)

        self.assertEqual(y, Young([[1,2,6],[3,4,7],[5]]))

class TestSkew(unittest.TestCase):
    # test of skew tableaux
    def test_skew1(self): 
        t = Young([[1,1,2],[2],[3]])
        y = Young([[1,1],[2]])
        skew = t/y
        self.assertEqual(skew,
                [[None, None, 2],[None], [3]])

    def test_skew2(self): 
        t = Young([[1,2,2,3],[2,3,5,5],[4,4,6],[5,6]])
        y = Young([[1,3],[2]])
        skew = t/y
        self.assertEqual(skew,
                [[None, None, 2, 3],[None, 3,5,5], [4,4,6],[5,6]])

def test_main(verbose=None):
    testclasses = (
            TestYoung,
            TestBump,
            TestSkew,
            )

    test_support.run_unittest(*testclasses)

if __name__ == '__main__':
    test_main()
